/*  gtklepin - a gtk+ notebook
 *
 *  Copyright (C) 20002  Luc Dufresne <luc@ordiluc.net>
 *                       26, rue des Comices
 *                       59650 Villeneuve d'Ascq
 *                       FRANCE
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See the file COPYING
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef _WIN32
#include "win32.h"
#include <direct.h>
#else
#include <unistd.h>
#include <getopt.h>
#include <time.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "boulot.h"
#include "misc.h"
#include "options.h"
#include "message.h"
#include "callbacks.h"

/* ar archive */

#define ARMAG   "!<arch>\n"     /* String that begins an archive file.  */
#define ARFMAG  "`\n"           /* String in ar_fmag at end of each header.  */
#define ARUID	"1000  "
#define ARGID	"1000  "
#define ARMODE	"100664  "

struct ar_hdr
  {
    char ar_name[16];           /* Member file name, sometimes / terminated. */
    char ar_date[12];           /* File date, decimal seconds since Epoch.  */
    char ar_uid[6], ar_gid[6];  /* User and group IDs, in ASCII decimal.  */
    char ar_mode[8];            /* File mode, in ASCII octal.  */
    char ar_size[10];           /* File size, in ASCII decimal.  */
    char ar_fmag[2];            /* Always contains ARFMAG.  */
  };

/* end ar */

struct progdata data;
static gint check_write_file (const gchar *file); 

#ifdef DEBUG
static void debug_print (gpointer data, gpointer a);
static void debug (const gchar *str);
#endif

static void wait_answer (); 

#ifndef _WIN32

#define SLEEP_TIME 100

static void version () {
        g_print(""PACKAGE" "VERSION" http://ordiluc.net/"PACKAGE"/\n");
        g_print("Copyright (c) 2002 Luc Dufresne <luc@ordiluc.net>\n");
}

static void usage () {
        version();
        g_print("\n");
        g_print(_("usage: "PACKAGE" [OPTIONS] [FILE_TO_OPEN]\n"));
        g_print(_("  -h, --help\t\t\tprints this message\n"));
        g_print(_("  -v, --version\t\t\tprints version information\n"));
}

void get_options (int argc, char *argv[]) {
	gint c,opind;
	extern int optind;
	static struct option long_options[] = {
                {"help", 0, 0, 'h'},
                {"version", 0, 0, 'v'},
                {0, 0, 0, 0}
        };

	while(1)
        {
                opind = 0;
                c = getopt_long (argc, argv, "hv", long_options, &opind);
                if (c == -1)
                break;
                switch(c)
                        {
                                case 0:
                                        break;
                                case 'h':
                                        usage();
                                        exit(0);
                                        break;
                                case 'v':
                                        version();
                                        exit(0);
                                        break;
                                default:
                                        usage();
                                        exit(1);
                                        break;
                        }
        }
	data.file_to_open = argv[optind];
}

#endif

void init (GtkWidget* window) {
	struct datastruct *data_s;
	GtkNotebook *notebook;
	
	data.main_window = window;
	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");

	data_s = g_malloc (sizeof (struct datastruct));
	data_s->label = (GtkLabel *) lookup_widget ((GtkWidget *)notebook, "label");
	data_s->lower = (GtkButton *) lookup_widget ((GtkWidget *)notebook, "lower_button");
	data_s->raise = (GtkButton *) lookup_widget ((GtkWidget *)notebook, "raise_button");
	data_s->close = (GtkButton *) lookup_widget ((GtkWidget *)notebook, "close_button");
	data_s->text = (GtkEditable *) lookup_widget ((GtkWidget *)notebook, "text");
	data_s->statusbar = (GtkStatusbar *) lookup_widget ((GtkWidget *)notebook, "statusbar");  

	data.to_destroy = NULL;
	data.nb_data = NULL;
	data.nb_data = g_list_append(data.nb_data, data_s);

	gtk_text_set_word_wrap ((GtkText *) data_s->text, 1);
	gtk_window_set_title (GTK_WINDOW (data.main_window), ""PACKAGE" "VERSION"");	

	read_options (data.main_window, data.nb_data);
	open_file ("");
	set_sensitive_button ();
}

static void set_title (const gchar *fichier) {
	GString *title;

	title = g_string_new ("");
	g_string_sprintf (title, ""PACKAGE" "VERSION" - %s" , fichier);
	gtk_window_set_title (GTK_WINDOW (data.main_window), title->str);
	g_string_free (title, TRUE);
}


static struct datastruct * find_data_num (gint page_num) {
	GList *list;

	list = g_list_nth(data.nb_data, page_num);
	return list->data;
}

static struct datastruct * find_data () {
	gint cur_pos;
	GtkNotebook *notebook;

	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	cur_pos = gtk_notebook_get_current_page (notebook);
	return find_data_num (cur_pos);
}

void switch_page (gint page_num) {
	struct datastruct *data_s;
	
	data.saved = 0;
	if (g_list_length(data.nb_data) > 1) {
		data_s = find_data_num (page_num);
		gtk_widget_grab_focus ((GtkWidget *) data_s->text);
	}
}

static GString * get_title (GtkEditable *text) {
	gint length;
	gchar *str;
	GString *buf;
	
	length = gtk_text_get_length((GtkText *) text);
	str = gtk_editable_get_chars(text, 0, (length > 20) ? 20 : length);
	if (str[0] != '\n') {
		buf = g_string_new (strtok(str,"\n"));
		if (buf->len == 20) g_string_append (buf, "...");
	} else {
		buf = g_string_new ("");
	}
	g_free (str);
	return buf;
}

void maj_label () {
	struct datastruct *data_s;
        struct tm *t;
        char time_str[1024];
	GString *buf;

#ifdef DEBUG	
	debug ("maj_label");
#endif
	data.saved = 0;
	data_s = find_data ();
	buf = get_title (data_s->text);
	gtk_label_set_text (data_s->label, buf->str);
	g_string_free (buf, TRUE);

	data_s->modification = time (&data_s->modification );
        t = localtime (&data_s->modification );
        strftime (time_str, 1024, "%c", t);
	gtk_label_set_text ((GtkLabel *) data_s->statusbar->label, time_str);
}

static void set_entry (GtkMenuItem *menuitem, gpointer user_data) {
	struct datastruct *nb_data;
	GtkNotebook *notebook;
	GList *list;
	gint pos;
	
	nb_data = (struct datastruct *) user_data;
	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	pos = 0;
	list = data.nb_data;
	while (list->data != nb_data) {
		pos++;
		list = list->next;
	}
	gtk_notebook_set_page(notebook, pos);
}

static void foreach_menu_window (gpointer d, gpointer a) {
	struct datastruct *nb_data;
	GtkWidget *new;
	GtkWidget *window_menu;
	GString *buf;	

	nb_data = (struct datastruct *) d;
	
	buf = get_title (nb_data->text);
	new = gtk_menu_item_new_with_label (buf->str);
	gtk_widget_set_name (new, buf->str);
	gtk_widget_ref (new);
	gtk_object_set_data_full (GTK_OBJECT (data.main_window), buf->str, new,
			(GtkDestroyNotify) gtk_widget_unref);
	g_string_free (buf, TRUE);
	
	data.to_destroy = g_list_append (data.to_destroy, new);
	
	gtk_widget_show (new);
	window_menu = lookup_widget (data.main_window, "window_menu");
	gtk_container_add (GTK_CONTAINER (window_menu), new);

	gtk_signal_connect (GTK_OBJECT (new), "activate",
			GTK_SIGNAL_FUNC (set_entry),
			nb_data);
}

void make_menu_window () {
	GList *list;

	/* clear the old menu */
	list = g_list_last (data.to_destroy);
	while (data.to_destroy != NULL) {
		gtk_widget_destroy ((GtkWidget *) list->data);
		data.to_destroy = g_list_remove (data.to_destroy, list->data);
		list = list->prev;
	}
	
	/* create the new menu */
	g_list_foreach (data.nb_data, foreach_menu_window, NULL);
}

void clear_text () {
	gint length;
	struct datastruct *data_s;

	data_s = find_data ();
	length = gtk_text_get_length((GtkText *)data_s->text);
	gtk_editable_delete_text(data_s->text,0,length);
}

struct save_struct {
	FILE *fichier;
	gint i, current;
};

static void save_foreach (gpointer data, gpointer a) {
	struct datastruct *nb_data;
	struct save_struct *save_s;
	gint length;
	gchar *str;
	
	nb_data = (struct datastruct *) data;
	save_s = (struct save_struct *) a;

	length = gtk_text_get_length((GtkText *) nb_data->text);
	
	if (save_s->i != save_s->current) 
		fprintf (save_s->fichier, "%-16i%-12li"ARUID""ARGID""ARMODE"%-10i"ARFMAG"", save_s->i, nb_data->modification, length);
	else
		fprintf (save_s->fichier, "%-16s%-12li"ARUID""ARGID""ARMODE"%-10i"ARFMAG"", "d", nb_data->modification, length);		

	str = gtk_editable_get_chars(nb_data->text, 0, length);
	fprintf (save_s->fichier, "%s", str);
	if (length %  2) fprintf (save_s->fichier, "\n"); /* obligatoire pour avoir une archive lisible avec ar */
	g_free (str);
	
	(save_s->i)++;
}

static void do_save (FILE *fichier) {
	struct save_struct save_s;
	GtkNotebook *notebook;

	fprintf (fichier, ARMAG);
	save_s.i = 0;
	save_s.fichier = fichier;
	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	save_s.current = gtk_notebook_get_current_page (notebook);
	g_list_foreach (data.nb_data, save_foreach, &save_s);
	fclose (fichier);
	data.saved = 1;
}

gint save_as () {
	gchar *str;
	FILE *fichier;
	
	data.status = 0;
	data.file_window = create_fileselection ();
	gtk_widget_show (data.file_window);
	wait_answer ();

	if (data.status == 1) {
		str = g_strdup (gtk_file_selection_get_filename ((GtkFileSelection *) data.file_window));
		gtk_widget_destroy (data.file_window);
		if (!check_write_file (str)) return 0;
		fichier = fopen (str, "w");
		if (fichier == NULL) {
			display_message (_("error"),_("Unable to write the file"));
			return 0;
		}
		do_save (fichier);
		g_string_sprintf(options.LastOpenedFile, "%s", str);
		save_options ();
		set_title (str);
		g_free (str);
		return 1;
	} else gtk_widget_destroy (data.file_window);
	return 0;
}

static void backup (const gchar *filename) {
	gchar *backup_filename;

	backup_filename = g_strdup_printf ("%s~", filename);
	rename (filename, backup_filename);
	g_free (backup_filename);
}

gint save () {
	FILE *fichier;

	if (options.LastOpenedFile->str == NULL || 0 == strcmp (options.LastOpenedFile->str, "")) {
		return save_as ();
	}

	if (options.Backup) backup (options.LastOpenedFile->str);
	fichier = fopen (options.LastOpenedFile->str, "w");
	if (fichier == NULL) {
		display_message (_("error"),_("Unable to write the file"));
		return 0;
	}
	do_save (fichier);
	return 1;
}

static void ask_save () {
	data.ask_window = create_dialog_save ();
	gtk_widget_grab_default (lookup_widget (data.ask_window, "button_yes"));
	gtk_widget_show (data.ask_window);
}

static void really_quit () {
	save_options ();
	gtk_main_quit ();
}

static void wait_answer () {
	while (data.status == 0) {
#ifndef _WIN32
		usleep(SLEEP_TIME);
#endif
		while (gtk_events_pending()) gtk_main_iteration();
	}
}

gboolean quit () {
	if (!data.saved) {
		data.status = 0;
		while (data.status == 0 || data.status == 4) {
			data.status = 0;
			ask_save ();
			wait_answer ();
		}
		if (data.status != 3) {
			really_quit ();
			return FALSE;
		}
		return TRUE;
	} else {
		really_quit ();
		return FALSE;
	}
}

void save_cancel () {
	gtk_widget_destroy(data.ask_window);
	data.status = 3;
}

void save_yes () {
	gtk_widget_destroy(data.ask_window);
	if (!save ()) data.status = 4;
	else data.status = 1;
}

void save_no () {
	gtk_widget_destroy(data.ask_window);
	data.status = 2;
}

static void sensitive_on (gpointer data, gpointer a) {
	struct datastruct *nb_data;
	nb_data = (struct datastruct *) data;
	gtk_widget_set_sensitive ((GtkWidget *) nb_data->lower, 1);
	gtk_widget_set_sensitive ((GtkWidget *) nb_data->raise, 1);
	gtk_widget_set_sensitive ((GtkWidget *) nb_data->close, 1);
}

void set_sensitive_button () {
	struct datastruct *data_s;

	data_s = data.nb_data->data;
	if (g_list_length (data.nb_data) == 1) {
		gtk_widget_set_sensitive ((GtkWidget *) data_s->lower, 0);
		gtk_widget_set_sensitive ((GtkWidget *) data_s->raise, 0);
		gtk_widget_set_sensitive ((GtkWidget *) data_s->close, 0);
	} else {
		g_list_foreach (data.nb_data, sensitive_on, NULL);
		gtk_widget_set_sensitive ((GtkWidget *) data_s->raise, 0);
		data_s = g_list_last (data.nb_data) -> data;
		gtk_widget_set_sensitive ((GtkWidget *) data_s->lower, 0);
	}
}

void add_entry (gint pos) {
	GtkNotebook *notebook;
	GtkWidget *vbox2, *label, *scrolledwindow, *text, *statusbar, *hbox3, *toolbar1, *lower_button, *raise_button, *close_button, *tmp_toolbar_icon;
	struct datastruct *data_s;
	gint cur_pos, true_pos;

	data.saved = 0;
	
	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_set_name (vbox2, "vbox2");
	gtk_widget_ref (vbox2);
	gtk_object_set_data_full (GTK_OBJECT (data.main_window), "vbox2", vbox2,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox2);

	scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_set_name (scrolledwindow, "scrolledwindow");
	gtk_widget_ref (scrolledwindow);
	gtk_object_set_data_full (GTK_OBJECT (data.main_window), "scrolledwindow", scrolledwindow,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (scrolledwindow);
	gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	text = gtk_text_new (NULL, NULL);
	gtk_widget_set_name (text, "text");
	gtk_widget_ref (text);
	gtk_object_set_data_full (GTK_OBJECT (data.main_window), "text", text,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (text);
	gtk_container_add (GTK_CONTAINER (scrolledwindow), text);
	gtk_text_set_editable (GTK_TEXT (text), TRUE);

	gtk_text_set_word_wrap ((GtkText *) text, 1);
	gtk_text_set_line_wrap ((GtkText *) text, options.LineWarp);
	
	statusbar = gtk_statusbar_new ();
	gtk_widget_set_name (statusbar, "statusbar");
	gtk_widget_ref (statusbar);
	gtk_object_set_data_full (GTK_OBJECT (data.main_window), "statusbar", statusbar,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (statusbar);
	gtk_box_pack_start (GTK_BOX (vbox2), statusbar, FALSE, FALSE, 0);

	hbox3 = gtk_hbox_new (FALSE, 0);
	gtk_widget_set_name (hbox3, "hbox3");
	gtk_widget_ref (hbox3);
	gtk_object_set_data_full (GTK_OBJECT (data.main_window), "hbox3", hbox3,
			(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox3);

	label = gtk_label_new ("");
	gtk_widget_set_name (label, "label");
	gtk_widget_ref (label);
	gtk_object_set_data_full (GTK_OBJECT (data.main_window), "label", label,
			(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (hbox3), label, TRUE, TRUE, 0);

	toolbar1 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
	gtk_widget_set_name (toolbar1, "toolbar1");
	gtk_widget_ref (toolbar1);
	gtk_object_set_data_full (GTK_OBJECT (data.main_window), "toolbar1", toolbar1,
			(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (toolbar1);
	gtk_box_pack_start (GTK_BOX (hbox3), toolbar1, FALSE, FALSE, 0);
	gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar1), GTK_RELIEF_NONE);

	tmp_toolbar_icon = create_pixmap (data.main_window, "down.xpm");
	lower_button = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
			GTK_TOOLBAR_CHILD_BUTTON,
			NULL,
			_("lower"),
			NULL, NULL,
			tmp_toolbar_icon, NULL, NULL);
	gtk_widget_set_name (lower_button, "lower_button");
	gtk_widget_ref (lower_button);
	gtk_object_set_data_full (GTK_OBJECT (data.main_window), "lower_button", lower_button,
			(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (lower_button);

	tmp_toolbar_icon = create_pixmap (data.main_window, "up.xpm");
	raise_button = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
			GTK_TOOLBAR_CHILD_BUTTON,
			NULL,
			_("raise"),
			_("Raise Entry"), NULL,
			tmp_toolbar_icon, NULL, NULL);
	gtk_widget_set_name (raise_button, "raise_button");
	gtk_widget_ref (raise_button);
	gtk_object_set_data_full (GTK_OBJECT (data.main_window), "raise_button", raise_button,
			(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (raise_button);

	tmp_toolbar_icon = create_pixmap (data.main_window, "close.xpm");
	close_button = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
			GTK_TOOLBAR_CHILD_BUTTON,
			NULL,
			_("close"),
			_("Close Entry"), NULL,
			tmp_toolbar_icon, NULL, NULL);
	gtk_widget_set_name (close_button, "close_button");
	gtk_widget_ref (close_button);
	gtk_object_set_data_full (GTK_OBJECT (data.main_window), "close_button", close_button,
			(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (close_button);

	data_s = g_malloc (sizeof (struct datastruct));
	data_s->label = (GtkLabel *) label;
	data_s->lower = (GtkButton *) lower_button;
	data_s->raise = (GtkButton *) raise_button;
	data_s->close = (GtkButton *) close_button;
	data_s->text = (GtkEditable *) text;
	data_s->statusbar = (GtkStatusbar *) statusbar;

	gtk_signal_connect (GTK_OBJECT (text), "changed",
	                    GTK_SIGNAL_FUNC (maj_label),
	                    NULL);

	gtk_signal_connect (GTK_OBJECT (lower_button), "clicked",
			GTK_SIGNAL_FUNC (on_lower_button_clicked),
			NULL);
	gtk_signal_connect (GTK_OBJECT (raise_button), "clicked",
			GTK_SIGNAL_FUNC (on_raise_button_clicked),
			NULL);
	gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
			GTK_SIGNAL_FUNC (on_close_button_clicked),
			NULL);

	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");

	if (pos == 0) {
		data.nb_data = g_list_append (data.nb_data, data_s);
		
		gtk_notebook_append_page(notebook,vbox2,hbox3);
		gtk_notebook_set_page(notebook, g_list_length(data.nb_data) - 1);
	} else {
		cur_pos = gtk_notebook_get_current_page (notebook);
		true_pos = 0;
		switch (pos) {
			case -1:
				true_pos = cur_pos;
				break;
			case 1:
				true_pos = cur_pos +1;
				break;
		}
	
		data.nb_data = g_list_insert (data.nb_data, data_s, true_pos);

		gtk_notebook_insert_page(notebook,vbox2,hbox3,true_pos);
		gtk_notebook_set_page(notebook, true_pos);
	}
}

static void delete_entry_nb (gint i) {
	GtkNotebook *notebook;
	GList *list;
	
	list = g_list_nth(data.nb_data, i);
#ifdef DEBUG
	g_print ("\n\n\n  ### to delete (pos %i) ###\n", i);
	debug_print (list->data, NULL);
#endif	
	g_free (list->data);
	data.nb_data = g_list_remove (data.nb_data, list->data);
	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	gtk_notebook_remove_page (notebook, i);
	data.saved = 0;
#ifdef DEBUG
	debug ("delete_entry");
#endif

}

static void delete_entry_nc () {
	gint cur_pos;
	GtkNotebook *notebook;	

	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	cur_pos = gtk_notebook_get_current_page (notebook);
	delete_entry_nb (cur_pos);
}

void delete_entry () {
	if (g_list_length(data.nb_data) == 1) return;
	delete_entry_nc ();
}

void close_entry (GtkButton *button_pressed) {
	gint i;
	GList *list;
	struct datastruct *data_s;

	list = data.nb_data;
	data_s = (struct datastruct *) list->data;
	i = 0;
	while (data_s->close != button_pressed) {
		i++;
		list = list->next;
		data_s = (struct datastruct *) list->data;
	}
	delete_entry_nb (i);
}

void line_wrap_callback (gboolean wrap) {
	line_wrap (wrap, data.nb_data);
}

static void move_entry_nb (gint entry, gint mov) {
	GtkNotebook *notebook;
	GList *list;
	struct datastruct *data_s;
	
	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	gtk_notebook_reorder_child (notebook, gtk_notebook_get_nth_page (notebook, entry), entry + mov);

	list = g_list_nth(data.nb_data, entry);
	data_s = list->data;
	data.nb_data = g_list_remove (data.nb_data, data_s);
#ifdef DEBUG
	g_print ("\n\n\n  ### to move (pos %i -> %i): ###\n", entry, entry + mov);
	debug_print (data_s, NULL);
#endif	
	data.nb_data = g_list_insert (data.nb_data, data_s, entry + mov);
#ifdef DEBUG
	debug ("move_entry");
#endif
	data.saved = 0;
}

void move_entry (gint i) {
	GtkNotebook *notebook;
	gint cur_pos;
	
	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	cur_pos = gtk_notebook_get_current_page (notebook);
	if (cur_pos + i == -1) return;
	move_entry_nb (cur_pos, i);
}

static gint find_entry_to_move (GtkButton *button_pressed, gint mov) {
	gint i;
	GList *list;
	struct datastruct *data_s;

	list = data.nb_data;
	data_s = (struct datastruct *) list->data;
	i = 0;
	for (;;) {
		switch (mov) {
			case 1:
				if (data_s->lower == button_pressed)
					return i;
			case -1:
				if (data_s->raise == button_pressed)
					return i;
		}
		i++;
		list = list->next;
		data_s = (struct datastruct *) list->data;
	}
	return i;
}

void raise_entry_callback (GtkButton *button_pressed) {
	move_entry_nb (find_entry_to_move (button_pressed, -1), -1);
}

void lower_entry_callback (GtkButton *button_pressed) {
	move_entry_nb (find_entry_to_move (button_pressed, 1), 1);
}

void file_cancel () {
	data.status = 3;	
}

void file_ok () {
	data.status = 1;
}

void import_text_file () {
	gchar *str;
	FILE *fichier;
	GString *buf;
	gint pos=0;
	struct datastruct *data_s;

	data.status = 0;
	data.file_window = create_fileselection ();
	gtk_widget_show (data.file_window);
	wait_answer ();

	if (data.status == 1) {
		str = g_strdup (gtk_file_selection_get_filename ((GtkFileSelection *) data.file_window));
		gtk_widget_destroy (data.file_window);
		fichier = fopen (str, "r");
		g_free (str);
		if (fichier == NULL) {
			display_message (_("Error"),_("Unable to read this file"));
			return;
		}

		add_entry (0);
		data_s = find_data ();

		gtk_text_freeze((GtkText *)data_s->text);
		while(!feof(fichier)) {
	                buf=getline(fichier);
			gtk_editable_insert_text(data_s->text, buf->str, buf->len, &pos);
			g_string_free(buf,TRUE);
		}
		gtk_text_thaw ((GtkText *)data_s->text);
		fclose (fichier);
	} else gtk_widget_destroy (data.file_window);
}

static gint check_write_file (const gchar *file) {
	struct stat buf;

	if (-1 == stat (file, &buf)) {
		return 1;
	} else {
#ifdef _WIN32
		if (S_IFDIR & buf.st_mode) {
#else
		if S_ISDIR(buf.st_mode) {
#endif
			display_message (_("error"),_("This is a directory"));
			return 0;
		}
		if (1 == display_ask (_("error"), _("This file already exists\nShould I overwrite it ?"))) return 1;
	}
	return 0;
}

void export_current_entry () {
	struct datastruct *data_s;
	gchar *str;
	FILE *fichier;

	data.status = 0;
	data.file_window = create_fileselection ();
	gtk_widget_show (data.file_window);
	wait_answer ();

	if (data.status == 1) {
		str = g_strdup (gtk_file_selection_get_filename ((GtkFileSelection *) data.file_window));
		gtk_widget_destroy (data.file_window);
		if (!check_write_file (str)) return;
		fichier = fopen (str, "w");
		g_free (str);
		if (fichier == NULL) {
			display_message (_("error"),_("Unable to write the file"));
			return;
		}
		
		data_s = find_data ();
		str = gtk_editable_get_chars(data_s->text, 0, gtk_text_get_length((GtkText *) data_s->text));
		fprintf (fichier, "%s", str);
		g_free (str);
		fclose (fichier);
	} else gtk_widget_destroy (data.file_window);
}

static gint check_directory (const gchar *file) {
	struct stat buf;
	
	if (-1 == stat (file, &buf)) {
		return 1;
	} else {
		display_message (_("error"),_("Something with that name already exists"));
		return 0;
	}
	return 0;
}

struct export_struct {
	gchar *dir;
	gint i;
	gboolean error;
};

static void export_foreach (gpointer data, gpointer a) {
	struct datastruct *nb_data;
	struct export_struct *export_s;
	gint length;
	gchar *str;
	FILE *fichier;
	GString *string;

	nb_data = (struct datastruct *) data;
	export_s = (struct export_struct *) a;

	string = g_string_new("");
	g_string_sprintf(string,"%s/%i", export_s->dir, export_s->i);

	fichier = fopen (string->str, "w");
	if (fichier == NULL) export_s->error = 1;
	length = gtk_text_get_length((GtkText *) nb_data->text);
	str = gtk_editable_get_chars(nb_data->text, 0, length);
	fprintf (fichier, "%s", str);	

	g_free (str);
	fclose (fichier);	
	g_string_free (string, TRUE);
	(export_s->i)++;	
}

void export_directory () {
	gchar *str;
	struct export_struct export_s;	
	
	data.status = 0;
	data.file_window = create_fileselection ();
	gtk_widget_show (data.file_window);
	wait_answer ();

	if (data.status == 1) {
		str = g_strdup (gtk_file_selection_get_filename ((GtkFileSelection *) data.file_window));
		gtk_widget_destroy (data.file_window);
		if (!check_directory (str)) return;
#ifdef _WIN32
		if (-1 == mkdir (str)) {
#else
		if (-1 == mkdir (str,S_IRWXU)) {
#endif
			g_free (str);
			display_message (_("error"),_("Unable to create this directory"));
			return;
		}
		export_s.i = 0;
		export_s.dir = str;
		export_s.error = 0;
		g_list_foreach (data.nb_data, export_foreach, &export_s);
		if (export_s.error) display_message(_("error"),_("Could not write everything"));
		g_free (str);
	} else gtk_widget_destroy (data.file_window);
}

static void create_new_nc () {
	/* delete everything */
	while (g_list_length(data.nb_data) != 0) delete_entry_nc ();

	/* create a new entry */
	add_entry (0);

	data.saved = 1;
	gtk_window_set_title (GTK_WINDOW (data.main_window), ""PACKAGE" "VERSION"");
	g_string_assign(options.LastOpenedFile, "");
	save_options ();
}

void create_new () {
	data.status = 0;
	if (!data.saved) {
		while (data.status == 0 || data.status == 4) {
			data.status = 0;
			ask_save ();
			wait_answer ();
		}
		if (data.status == 3) return;
	}
	create_new_nc ();
}

void open_file (gchar *fichier_a_ouvrir) {
	FILE *fichier;
	gchar buf[sizeof(struct ar_hdr) + 1];
	gchar tmp[sizeof(struct ar_hdr) + 1];
	gchar tmp2[sizeof(ARFMAG)];
	gint size, i, pos, page;
	gchar *str;
	struct datastruct *data_s;
	GtkNotebook *notebook;
	struct tm *t;
	char time_str[1024];
	time_t date;
	gchar *fichier_a_ouvrir_reel;
	GString *err;
	
	if (0 == strcmp (fichier_a_ouvrir, "")) {   /* au dmarrage du programme */
		if (NULL != data.file_to_open  &&
				0 != strcmp (data.file_to_open, "")) {
			fichier_a_ouvrir_reel = data.file_to_open;
#ifdef DEBUG
			g_print ("command line\n");
#endif
		}
		else {
			if (options.OpenLast &&
					options.LastOpenedFile->str != NULL &&
					0 != strcmp (options.LastOpenedFile->str, "")) {
				fichier_a_ouvrir_reel = options.LastOpenedFile->str;
#ifdef DEBUG
				g_print ("last opened file\n");
#endif
			}
			else {
#ifdef DEBUG
				g_print ("nothing to open\n");
#endif
				data.saved = 1;
				return;
			}
		}
	} else {
		/* delete everything */
		while (g_list_length(data.nb_data) != 0) delete_entry_nc ();

		/* create a new entry */
		add_entry (0);

		fichier_a_ouvrir_reel = fichier_a_ouvrir;
	}
#ifdef DEBUG
	g_print ("file to open: --%s--\n", fichier_a_ouvrir_reel);
#endif
	fichier = fopen (fichier_a_ouvrir_reel, "r");
	if (fichier == NULL) {
		err = g_string_new ("");
		g_string_sprintf (err, _("Unable to read the file: \"%s\""), fichier_a_ouvrir_reel);
		display_message (_("error"), err->str);
		g_string_free (err, TRUE);
		data.saved = 1;
		gtk_window_set_title (GTK_WINDOW (data.main_window), ""PACKAGE" "VERSION"");
		return;		
	}

	if (NULL == fgets (buf, sizeof(ARMAG) +1, fichier) ||
			0 != strcmp (buf, ARMAG)) {
		err = g_string_new ("");
		g_string_sprintf (err, _("The file \"%s\" is not valid"), fichier_a_ouvrir_reel);
		display_message (_("error"), err->str);
		g_string_free (err, TRUE);
		data.saved = 1;
		gtk_window_set_title (GTK_WINDOW (data.main_window), ""PACKAGE" "VERSION"");
		return;
	}

	i = 0;
	page = 0;
	while (!feof(fichier)) {
		if(NULL == fgets (buf, sizeof(struct ar_hdr) + 1, fichier)) break;

		strncpy (tmp2, &buf[58],2);
		tmp2[2] = '\0';
		if (0 != strcmp (tmp2, ARFMAG)) {
			display_message (_("error"),_("Error while reading the file\nThe file might not be valid"));
			create_new_nc ();
			return;
		}
	
		strncpy(tmp, buf, 1);
		tmp[1] = '\0';
		if (0 == strcmp(tmp,"d")) page = i;
		
		strncpy (tmp, &buf[16],12);
		tmp[12] = '\0';
		date = atoi (tmp);

		strncpy (tmp, &buf[48],10);
		tmp[12] = '\0';
		size = atoi (tmp);
		if (i) add_entry (0);

		data_s = find_data ();
		if(size > 0) {
			str = g_malloc(size + 1);
			if (1 != fread(str, size, 1, fichier)) {
				display_message (_("error"),_("Error while reading the file\nThe file might not be valid"));
				g_free (str);
				create_new_nc ();
				return;
				}
			pos = 0;
			gtk_text_freeze((GtkText *)data_s->text);	
			gtk_editable_insert_text(data_s->text, str, size, &pos);
			gtk_text_thaw ((GtkText *)data_s->text);
			if(size % 2) getc(fichier);
			g_free (str);
		}

		data_s->modification = date;
		t = localtime (&data_s->modification);
	        strftime (time_str, 1024, "%c", t);
		gtk_label_set_text ((GtkLabel *) data_s->statusbar->label, time_str);
		
		i++;
	}
	fclose (fichier);

	notebook = (GtkNotebook *) lookup_widget (data.main_window, "notebook");
	gtk_notebook_set_page(notebook, page);
	data.saved = 1;

	fichier_a_ouvrir_reel = g_strdup (fichier_a_ouvrir_reel);
	g_string_sprintf(options.LastOpenedFile, "%s", fichier_a_ouvrir_reel);
	save_options ();
	set_title (fichier_a_ouvrir_reel);
	g_free (fichier_a_ouvrir_reel);
}

void do_open_file () {
	gchar *str;

	if (!data.saved) {
		data.status = 0;
		while (data.status == 0 || data.status == 4) {
			data.status = 0;
			ask_save ();
			wait_answer ();
		}
		if (data.status == 3) return;
	}
	
	data.status = 0;
	data.file_window = create_fileselection ();
	gtk_widget_show (data.file_window);
	wait_answer ();

	if (data.status == 1) {
		str = g_strdup (gtk_file_selection_get_filename ((GtkFileSelection *) data.file_window));
		gtk_widget_destroy (data.file_window);
		open_file (str);
		g_free (str);
	} else gtk_widget_destroy (data.file_window);
}

#ifdef DEBUG
static void debug_print (gpointer data, gpointer a) {
	struct datastruct *nb_data;
	gchar *str;

	nb_data = (struct datastruct *) data;
	str = gtk_editable_get_chars(nb_data->text, 0, gtk_text_get_length((GtkText *) nb_data->text));

	g_print ("----\n%s\n----\n", str);
	g_free (str);
}

static void debug (const gchar *str) {

	g_print ("\n\n\n  ### DEBUG: %s ###\n",str);
	g_list_foreach (data.nb_data, debug_print, NULL);
}
#endif
